from typing import Optional

import typer
from typing_extensions import Annotated


def main(accept: Annotated[Optional[bool], typer.Option("--accept/--reject")] = None):
    if accept is None:
        print("I don't know what you want yet")
    elif accept:
        print("Accepting!")
    else:
        print("Rejecting!")


if __name__ == "__main__":
    typer.run(main)



import typer
from typer.testing import CliRunner

runner = CliRunner()


def test_default_with_class_with_custom_eq():
    app = typer.Typer()

    from typer.models import ParamMeta

    class StupidClass:
        def __init__(self, a):
            self.a = a

        def __eq__(self, other) -> bool:
            if other is ParamMeta.empty:
                return True
            try:
                return self.a == other.a
            except Exception:
                return False

        def __ne__(self, other: object) -> bool:
            return not self.__eq__(other)

    @app.command()
    def cmd(val=StupidClass(42)):
        print(val)

    assert StupidClass(666) == ParamMeta.empty
    assert StupidClass(666) != StupidClass(1)

    result = runner.invoke(app)
    assert result.exit_code == 0, result.output
    assert "StupidClass" in result.output



import typer


def main(name: str, age: int = 20, height_meters: float = 1.89, female: bool = True):
    print(f"NAME is {name}, of type: {type(name)}")
    print(f"--age is {age}, of type: {type(age)}")
    print(f"--height-meters is {height_meters}, of type: {type(height_meters)}")
    print(f"--female is {female}, of type: {type(female)}")


if __name__ == "__main__":
    typer.run(main)



import typer


def main(
    name: str = typer.Argument(
        "Wade Wilson", help="Who to greet", show_default="Deadpoolio the amazing's name"
    ),
):
    print(f"Hello {name}")


if __name__ == "__main__":
    typer.run(main)



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.parameter_types.uuid import tutorial001 as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_main():
    result = runner.invoke(app, ["d48edaa6-871a-4082-a196-4daab372d4a1"])
    assert result.exit_code == 0
    assert "USER_ID is d48edaa6-871a-4082-a196-4daab372d4a1" in result.output
    assert "UUID version is: 4" in result.output


def test_invalid_uuid():
    result = runner.invoke(app, ["7479706572-72756c6573"])
    assert result.exit_code != 0
    assert (
        "Invalid value for 'USER_ID': '7479706572-72756c6573' is not a valid UUID"
        in result.output
    )


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



from importlib.machinery import ModuleSpec
from typing import Union
from unittest.mock import patch

import pytest
from typer._completion_classes import _sanitize_help_text


@pytest.mark.parametrize(
    "find_spec, help_text, expected",
    [
        (
            ModuleSpec("rich", loader=None),
            "help text without rich tags",
            "help text without rich tags",
        ),
        (
            None,
            "help text without rich tags",
            "help text without rich tags",
        ),
        (
            ModuleSpec("rich", loader=None),
            "help [bold]with[/] rich tags",
            "help with rich tags",
        ),
        (
            None,
            "help [bold]with[/] rich tags",
            "help [bold]with[/] rich tags",
        ),
    ],
)
def test_sanitize_help_text(
    find_spec: Union[ModuleSpec, None], help_text: str, expected: str
):
    with patch("importlib.util.find_spec", return_value=find_spec) as mock_find_spec:
        assert _sanitize_help_text(help_text) == expected
    mock_find_spec.assert_called_once_with("rich")



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.parameter_types.custom_types import tutorial001_an as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0


def test_parse_custom_type():
    result = runner.invoke(app, ["0", "--custom-opt", "1"])
    assert "custom_arg is <CustomClass: value=00>" in result.output
    assert "custom-opt is <CustomClass: value=11>" in result.output


def test_parse_custom_type_with_default():
    result = runner.invoke(app, ["0"])
    assert "custom_arg is <CustomClass: value=00>" in result.output
    assert "custom-opt is <CustomClass: value=FooFoo>" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer
from typing_extensions import Annotated


def main(config: Annotated[typer.FileTextWrite, typer.Option()]):
    config.write("Some config written by the app")
    print("Config written")


if __name__ == "__main__":
    typer.run(main)



from typing import Tuple

import typer


def main(user: Tuple[str, int, bool] = typer.Option((None, None, None))):
    username, coins, is_wizard = user
    if not username:
        print("No user provided")
        raise typer.Abort()
    print(f"The username {username} has {coins} coins")
    if is_wizard:
        print("And this user is a wizard!")


if __name__ == "__main__":
    typer.run(main)



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.parameter_types.number import tutorial002_an as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_invalid_id():
    result = runner.invoke(app, ["1002"])
    assert result.exit_code != 0
    assert (
        "Invalid value for 'ID': 1002 is not in the range 0<=x<=1000" in result.output
    )


def test_clamped():
    result = runner.invoke(app, ["5", "--rank", "11", "--score", "-5"])
    assert result.exit_code == 0
    assert "ID is 5" in result.output
    assert "--rank is 10" in result.output
    assert "--score is 0" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer

app = typer.Typer()


@app.command()
def found(name: str):
    print(f"Founding town: {name}")


@app.command()
def burn(name: str):
    print(f"Burning town: {name}")


if __name__ == "__main__":
    app()



import typer
from typing_extensions import Annotated

valid_completion_items = [
    ("Camila", "The reader of books."),
    ("Carlos", "The writer of scripts."),
    ("Sebastian", "The type hints guy."),
]


def complete_name(incomplete: str):
    for name, help_text in valid_completion_items:
        if name.startswith(incomplete):
            yield (name, help_text)


app = typer.Typer()


@app.command()
def main(
    name: Annotated[
        str, typer.Option(help="The name to say hi to.", autocompletion=complete_name)
    ] = "World",
):
    print(f"Hello {name}")


if __name__ == "__main__":
    app()



import typer


def main(
    name: str, lastname: str = typer.Option(..., prompt="Please tell me your last name")
):
    print(f"Hello {name} {lastname}")


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys
from pathlib import Path

from docs_src.commands.index import tutorial001 as mod

from ..utils import needs_bash, needs_linux, requires_completion_permission


@needs_bash
@needs_linux
def test_show_completion():
    result = subprocess.run(
        [
            "bash",
            "-c",
            f"'{sys.executable}' -m coverage run '{mod.__file__}' --show-completion",
        ],
        capture_output=True,
        encoding="utf-8",
        env={**os.environ, "SHELL": "/bin/bash", "_TYPER_COMPLETE_TESTING": "True"},
    )
    assert "_TUTORIAL001.PY_COMPLETE=complete_bash" in result.stdout


@needs_bash
@needs_linux
@requires_completion_permission
def test_install_completion():
    bash_completion_path: Path = Path.home() / ".bashrc"
    text = ""
    if bash_completion_path.is_file():  # pragma: no cover
        text = bash_completion_path.read_text()
    result = subprocess.run(
        [
            "bash",
            "-c",
            f"'{sys.executable}' -m coverage run '{mod.__file__}' --install-completion",
        ],
        capture_output=True,
        encoding="utf-8",
        env={**os.environ, "SHELL": "/bin/bash", "_TYPER_COMPLETE_TESTING": "True"},
    )
    new_text = bash_completion_path.read_text()
    bash_completion_path.write_text(text)
    assert "source" in new_text
    assert str(Path(".bash_completions/tutorial001.py.sh")) in new_text
    assert "completion installed in" in result.stdout
    assert "Completion will take effect once you restart the terminal" in result.stdout


def test_completion_invalid_instruction():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "sourcebash",
        },
    )
    assert result.returncode != 0
    assert "Invalid completion instruction." in result.stderr


def test_completion_source_bash():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_bash",
        },
    )
    assert (
        "complete -o default -F _tutorial001py_completion tutorial001.py"
        in result.stdout
    )


def test_completion_source_invalid_shell():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_xxx",
        },
    )
    assert "Shell xxx not supported." in result.stderr


def test_completion_source_invalid_instruction():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "explode_bash",
        },
    )
    assert 'Completion instruction "explode" not supported.' in result.stderr


def test_completion_source_zsh():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_zsh",
        },
    )
    assert "compdef _tutorial001py_completion tutorial001.py" in result.stdout


def test_completion_source_fish():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_fish",
        },
    )
    assert "complete --command tutorial001.py --no-files" in result.stdout


def test_completion_source_powershell():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_powershell",
        },
    )
    assert (
        "Register-ArgumentCompleter -Native -CommandName tutorial001.py -ScriptBlock $scriptblock"
        in result.stdout
    )


def test_completion_source_pwsh():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL001.PY_COMPLETE": "source_pwsh",
        },
    )
    assert (
        "Register-ArgumentCompleter -Native -CommandName tutorial001.py -ScriptBlock $scriptblock"
        in result.stdout
    )



from typing import Optional

import typer
from typing_extensions import Annotated

__version__ = "0.1.0"


def version_callback(value: bool):
    if value:
        print(f"Awesome CLI Version: {__version__}")
        raise typer.Exit()


def main(
    name: Annotated[str, typer.Option()] = "World",
    version: Annotated[
        Optional[bool], typer.Option("--version", callback=version_callback)
    ] = None,
):
    print(f"Hello {name}")


if __name__ == "__main__":
    typer.run(main)



import typer

image_desc = [
    ("alpine:latest", "latest alpine image"),
    ("alpine:hello", "fake image: for testing"),
    ("nvidia/cuda:10.0-devel-ubuntu18.04", ""),
]


def _complete(incomplete: str) -> str:
    for image, desc in image_desc:
        if image.startswith(incomplete):
            yield image, desc


app = typer.Typer()


@app.command()
def image(name: str = typer.Option(autocompletion=_complete)):
    typer.echo(name)


if __name__ == "__main__":
    app()



import logging
import os
import re
import subprocess
from functools import lru_cache
from http.server import HTTPServer, SimpleHTTPRequestHandler
from importlib import metadata
from pathlib import Path

import typer

logging.basicConfig(level=logging.INFO)

mkdocs_name = "mkdocs.yml"
en_docs_path = Path("")

app = typer.Typer()


@lru_cache
def is_mkdocs_insiders() -> bool:
    version = metadata.version("mkdocs-material")
    return "insiders" in version


@app.callback()
def callback() -> None:
    if is_mkdocs_insiders():
        os.environ["INSIDERS_FILE"] = "./mkdocs.insiders.yml"
    # For MacOS with insiders and Cairo
    os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = "/opt/homebrew/lib"


def generate_readme_content() -> str:
    en_index = en_docs_path / "docs" / "index.md"
    content = en_index.read_text("utf-8")
    match_pre = re.search(r"</style>\n\n", content)
    if not match_pre:
        raise RuntimeError("Couldn't find pre section (<style>) in index.md")
    frontmatter_end = match_pre.end()
    new_content = content[frontmatter_end:]
    # Remove content between <!-- only-mkdocs --> and <!-- /only-mkdocs -->
    new_content = re.sub(
        r"<!-- only-mkdocs -->.*?<!-- /only-mkdocs -->",
        "",
        new_content,
        flags=re.DOTALL,
    )
    return new_content


@app.command()
def generate_readme() -> None:
    """
    Generate README.md content from main index.md
    """
    typer.echo("Generating README")
    readme_path = Path("README.md")
    new_content = generate_readme_content()
    readme_path.write_text(new_content, encoding="utf-8")


@app.command()
def verify_readme() -> None:
    """
    Verify README.md content from main index.md
    """
    typer.echo("Verifying README")
    readme_path = Path("README.md")
    generated_content = generate_readme_content()
    readme_content = readme_path.read_text("utf-8")
    if generated_content != readme_content:
        typer.secho(
            "README.md outdated from the latest index.md", color=typer.colors.RED
        )
        raise typer.Abort()
    typer.echo("Valid README ✅")


@app.command()
def live(dirty: bool = False) -> None:
    """
    Serve with livereload a docs site for a specific language.

    This only shows the actual translated files, not the placeholders created with
    build-all.

    Takes an optional LANG argument with the name of the language to serve, by default
    en.
    """
    # Enable line numbers during local development to make it easier to highlight
    args = ["mkdocs", "serve", "--dev-addr", "127.0.0.1:8008"]
    if dirty:
        args.append("--dirty")
    subprocess.run(args, env={**os.environ, "LINENUMS": "true"}, check=True)


@app.command()
def build() -> None:
    """
    Build the docs.
    """
    insiders_env_file = os.environ.get("INSIDERS_FILE")
    print(f"Insiders file {insiders_env_file}")
    if is_mkdocs_insiders():
        print("Using insiders")
    print("Building docs")
    subprocess.run(["mkdocs", "build"], check=True)
    typer.secho("Successfully built docs", color=typer.colors.GREEN)


@app.command()
def serve() -> None:
    """
    A quick server to preview a built site.

    For development, prefer the command live (or just mkdocs serve).

    This is here only to preview the documentation site.

    Make sure you run the build command first.
    """
    typer.echo("Warning: this is a very simple server.")
    typer.echo("For development, use the command live instead.")
    typer.echo("This is here only to preview the documentation site.")
    typer.echo("Make sure you run the build command first.")
    os.chdir("site")
    server_address = ("", 8008)
    server = HTTPServer(server_address, SimpleHTTPRequestHandler)
    typer.echo("Serving at: http://127.0.0.1:8008")
    server.serve_forever()


if __name__ == "__main__":
    app()



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.parameter_types.number import tutorial003 as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_main():
    result = runner.invoke(app)
    assert result.exit_code == 0
    assert "Verbose level is 0" in result.output


def test_verbose_1():
    result = runner.invoke(app, ["--verbose"])
    assert result.exit_code == 0
    assert "Verbose level is 1" in result.output


def test_verbose_3():
    result = runner.invoke(app, ["--verbose", "--verbose", "--verbose"])
    assert result.exit_code == 0
    assert "Verbose level is 3" in result.output


def test_verbose_short_1():
    result = runner.invoke(app, ["-v"])
    assert result.exit_code == 0
    assert "Verbose level is 1" in result.output


def test_verbose_short_3():
    result = runner.invoke(app, ["-v", "-v", "-v"])
    assert result.exit_code == 0
    assert "Verbose level is 3" in result.output


def test_verbose_short_3_condensed():
    result = runner.invoke(app, ["-vvv"])
    assert result.exit_code == 0
    assert "Verbose level is 3" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



from enum import Enum
from pathlib import Path
from typing import Any, List, Optional, Tuple

import click
import pytest
import typer
from typer.testing import CliRunner

from .utils import needs_py310

runner = CliRunner()


def test_optional():
    app = typer.Typer()

    @app.command()
    def opt(user: Optional[str] = None):
        if user:
            print(f"User: {user}")
        else:
            print("No user")

    result = runner.invoke(app)
    assert result.exit_code == 0
    assert "No user" in result.output

    result = runner.invoke(app, ["--user", "Camila"])
    assert result.exit_code == 0
    assert "User: Camila" in result.output


@needs_py310
def test_union_type_optional():
    app = typer.Typer()

    @app.command()
    def opt(user: str | None = None):
        if user:
            print(f"User: {user}")
        else:
            print("No user")

    result = runner.invoke(app)
    assert result.exit_code == 0
    assert "No user" in result.output

    result = runner.invoke(app, ["--user", "Camila"])
    assert result.exit_code == 0
    assert "User: Camila" in result.output


def test_optional_tuple():
    app = typer.Typer()

    @app.command()
    def opt(number: Optional[Tuple[int, int]] = None):
        if number:
            print(f"Number: {number}")
        else:
            print("No number")

    result = runner.invoke(app)
    assert result.exit_code == 0
    assert "No number" in result.output

    result = runner.invoke(app, ["--number", "4", "2"])
    assert result.exit_code == 0
    assert "Number: (4, 2)" in result.output


def test_no_type():
    app = typer.Typer()

    @app.command()
    def no_type(user):
        print(f"User: {user}")

    result = runner.invoke(app, ["Camila"])
    assert result.exit_code == 0
    assert "User: Camila" in result.output


class SomeEnum(Enum):
    ONE = "one"
    TWO = "two"
    THREE = "three"


@pytest.mark.parametrize(
    "type_annotation",
    [List[Path], List[SomeEnum], List[str]],
)
def test_list_parameters_convert_to_lists(type_annotation):
    # Lists containing objects that are converted by Click (i.e. not Path or Enum)
    # should not be inadvertently converted to tuples
    expected_element_type = type_annotation.__args__[0]
    app = typer.Typer()

    @app.command()
    def list_conversion(container: type_annotation):
        assert isinstance(container, list)
        for element in container:
            assert isinstance(element, expected_element_type)

    result = runner.invoke(app, ["one", "two", "three"])
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "type_annotation",
    [
        Tuple[str, str],
        Tuple[str, Path],
        Tuple[Path, Path],
        Tuple[str, SomeEnum],
        Tuple[SomeEnum, SomeEnum],
    ],
)
def test_tuple_parameter_elements_are_converted_recursively(type_annotation):
    # Tuple elements that aren't converted by Click (i.e. Path or Enum)
    # should be recursively converted by Typer
    expected_element_types = type_annotation.__args__
    app = typer.Typer()

    @app.command()
    def tuple_recursive_conversion(container: type_annotation):
        assert isinstance(container, tuple)
        for element, expected_type in zip(container, expected_element_types):
            assert isinstance(element, expected_type)

    result = runner.invoke(app, ["one", "two"])
    assert result.exit_code == 0


def test_custom_parse():
    app = typer.Typer()

    @app.command()
    def custom_parser(
        hex_value: int = typer.Argument(None, parser=lambda x: int(x, 0)),
    ):
        assert hex_value == 0x56

    result = runner.invoke(app, ["0x56"])
    assert result.exit_code == 0


def test_custom_click_type():
    class BaseNumberParamType(click.ParamType):
        name = "base_integer"

        def convert(
            self,
            value: Any,
            param: Optional[click.Parameter],
            ctx: Optional[click.Context],
        ) -> Any:
            return int(value, 0)

    app = typer.Typer()

    @app.command()
    def custom_click_type(
        hex_value: int = typer.Argument(None, click_type=BaseNumberParamType()),
    ):
        assert hex_value == 0x56

    result = runner.invoke(app, ["0x56"])
    assert result.exit_code == 0



import typer
from typing_extensions import Annotated, Literal


def main(
    network: Annotated[Literal["simple", "conv", "lstm"], typer.Option()] = "simple",
):
    print(f"Training neural network of type: {network}")


if __name__ == "__main__":
    typer.run(main)



import sys

import typer
import typer.completion
from typer.testing import CliRunner

runner = CliRunner()


def test_rich_utils_click_rewrapp():
    app = typer.Typer(rich_markup_mode="markdown")

    @app.command()
    def main():
        """
        \b
        Some text

        Some unwrapped text
        """
        print("Hello World")

    @app.command()
    def secondary():
        """
        \b
        Secondary text

        Some unwrapped text
        """
        print("Hello Secondary World")

    result = runner.invoke(app, ["--help"])
    assert "Some text" in result.stdout
    assert "Secondary text" in result.stdout
    assert "\b" not in result.stdout
    result = runner.invoke(app, ["main"])
    assert "Hello World" in result.stdout
    result = runner.invoke(app, ["secondary"])
    assert "Hello Secondary World" in result.stdout


def test_rich_help_no_commands():
    """Ensure that the help still works for a Typer instance with no commands, but with a callback."""
    app = typer.Typer(help="My cool Typer app")

    @app.callback(invoke_without_command=True, no_args_is_help=True)
    def main() -> None:
        return None  # pragma: no cover

    result = runner.invoke(app, ["--help"])

    assert result.exit_code == 0
    assert "Show this message" in result.stdout


def test_rich_doesnt_print_None_default():
    app = typer.Typer(rich_markup_mode="rich")

    @app.command()
    def main(
        name: str,
        option_1: str = typer.Option(
            "option_1_default",
        ),
        option_2: str = typer.Option(
            ...,
        ),
    ):
        print(f"Hello {name}")
        print(f"First: {option_1}")
        print(f"Second: {option_2}")

    result = runner.invoke(app, ["--help"])
    assert "Usage" in result.stdout
    assert "name" in result.stdout
    assert "option-1" in result.stdout
    assert "option-2" in result.stdout
    assert result.stdout.count("[default: None]") == 0
    result = runner.invoke(app, ["Rick", "--option-2=Morty"])
    assert "Hello Rick" in result.stdout
    assert "First: option_1_default" in result.stdout
    assert "Second: Morty" in result.stdout


def test_rich_markup_import_regression():
    # Remove rich.markup if it was imported by other tests
    if "rich" in sys.modules:
        rich_module = sys.modules["rich"]
        if hasattr(rich_module, "markup"):
            delattr(rich_module, "markup")

    app = typer.Typer(rich_markup_mode=None)

    @app.command()
    def main(bar: str):
        pass  # pragma: no cover

    result = runner.invoke(app, ["--help"])
    assert "Usage" in result.stdout
    assert "BAR" in result.stdout



import typer

from .users import app as users_app
from .version import app as version_app

app = typer.Typer()

app.add_typer(version_app)
app.add_typer(users_app, name="users")


if __name__ == "__main__":
    app()



import typer
from typing_extensions import Annotated


def main(config: Annotated[typer.FileText, typer.Option(mode="a")]):
    config.write("This is a single line\n")
    print("Config line written")


if __name__ == "__main__":
    typer.run(main)



from typing import Optional

import typer

app = typer.Typer()


@app.command()
def main(name: str, city: Optional[str] = None):
    print(f"Hello {name}")
    if city:
        print(f"Let's have a coffee in {city}")


if __name__ == "__main__":
    app()



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.multiple_values.options_with_multiple_values import tutorial001 as mod

runner = CliRunner()
app = typer.Typer()
app.command()(mod.main)


def test_main():
    result = runner.invoke(app)
    assert result.exit_code != 0
    assert "No user provided" in result.output
    assert "Aborted" in result.output


def test_user_1():
    result = runner.invoke(app, ["--user", "Camila", "50", "yes"])
    assert result.exit_code == 0
    assert "The username Camila has 50 coins" in result.output
    assert "And this user is a wizard!" in result.output


def test_user_2():
    result = runner.invoke(app, ["--user", "Morty", "3", "no"])
    assert result.exit_code == 0
    assert "The username Morty has 3 coins" in result.output
    assert "And this user is a wizard!" not in result.output


def test_invalid_user():
    result = runner.invoke(app, ["--user", "Camila", "50"])
    assert result.exit_code != 0
    assert "Option '--user' requires 3 arguments" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



from enum import Enum

import typer
from typer.testing import CliRunner

app = typer.Typer(context_settings={"token_normalize_func": str.lower})


class User(str, Enum):
    rick = "Rick"
    morty = "Morty"


@app.command()
def hello(name: User = User.rick) -> None:
    print(f"Hello {name.value}!")


runner = CliRunner()


def test_enum_choice() -> None:
    # This test is only for coverage of the new custom TyperChoice class
    result = runner.invoke(app, ["--name", "morty"], catch_exceptions=False)
    assert result.exit_code == 0
    assert "Hello Morty!" in result.output

    result = runner.invoke(app, ["--name", "Rick"])
    assert result.exit_code == 0
    assert "Hello Rick!" in result.output

    result = runner.invoke(app, ["--name", "RICK"])
    assert result.exit_code == 0
    assert "Hello Rick!" in result.output



import typer
from typing_extensions import Annotated


def main(file: Annotated[typer.FileBinaryRead, typer.Option()]):
    processed_total = 0
    for bytes_chunk in file:
        # Process the bytes in bytes_chunk
        processed_total += len(bytes_chunk)
        print(f"Processed bytes total: {processed_total}")


if __name__ == "__main__":
    typer.run(main)



from typing import List, Optional

import typer
from typing_extensions import Annotated


def main(user: Annotated[Optional[List[str]], typer.Option()] = None):
    if not user:
        print(f"No provided users (raw input = {user})")
        raise typer.Abort()
    for u in user:
        print(f"Processing user: {u}")


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys

from typer.testing import CliRunner

from docs_src.options_autocompletion import tutorial004 as mod

runner = CliRunner()


def test_completion():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TUTORIAL004.PY_COMPLETE": "complete_zsh",
            "_TYPER_COMPLETE_ARGS": "tutorial004_aux.py --name ",
        },
    )
    assert '"Camila":"The reader of books."' in result.stdout
    assert '"Carlos":"The writer of scripts."' in result.stdout
    assert '"Sebastian":"The type hints guy."' in result.stdout


def test_1():
    result = runner.invoke(mod.app, ["--name", "Camila"])
    assert result.exit_code == 0
    assert "Hello Camila" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer
from typing_extensions import Annotated

app = typer.Typer(rich_markup_mode="rich")


@app.command(help="Say [bold red]hello[/bold red] to the user.")
def hello(
    user_1: Annotated[
        str,
        typer.Argument(help="The [bold]cool[/bold] name of the [green]user[/green]"),
    ],
    user_2: Annotated[str, typer.Argument(help="The world")] = "The World",
    force: Annotated[
        bool, typer.Option(help="Force the welcome [red]message[/red]")
    ] = False,
):
    print(f"Hello {user_1} and {user_2}")  # pragma: no cover


if __name__ == "__main__":
    app()



import typer


def main(
    name: str = typer.Argument(..., help="Who to greet"),
    lastname: str = typer.Argument(
        "", help="The last name", rich_help_panel="Secondary Arguments"
    ),
    age: str = typer.Argument(
        "", help="The user's age", rich_help_panel="Secondary Arguments"
    ),
):
    """
    Say hi to NAME very gently, like Dirk.
    """
    print(f"Hello {name}")


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys

from . import colon_example as mod


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--name", "DeadPool"],
        capture_output=True,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "DeadPool" in result.stdout


def test_completion_colon_bash_all():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_bash",
            "COMP_WORDS": "colon_example.py --name ",
            "COMP_CWORD": "2",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" in result.stdout


def test_completion_colon_bash_partial():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_bash",
            "COMP_WORDS": "colon_example.py --name alpine ",
            "COMP_CWORD": "2",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_bash_single():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_bash",
            "COMP_WORDS": "colon_example.py --name alpine:hell ",
            "COMP_CWORD": "2",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" not in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_zsh_all():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_zsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name ",
        },
    )
    assert "alpine\\\\:hello" in result.stdout
    assert "alpine\\\\:latest" in result.stdout
    assert "nvidia/cuda\\\\:10.0-devel-ubuntu18.04" in result.stdout


def test_completion_colon_zsh_partial():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_zsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine",
        },
    )
    assert "alpine\\\\:hello" in result.stdout
    assert "alpine\\\\:latest" in result.stdout
    assert "nvidia/cuda\\\\:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_zsh_single():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_zsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine:hell",
        },
    )
    assert "alpine\\\\:hello" in result.stdout
    assert "alpine\\\\:latest" not in result.stdout
    assert "nvidia/cuda\\\\:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_powershell_all():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_powershell",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name ",
            "_TYPER_COMPLETE_WORD_TO_COMPLETE": "",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" in result.stdout


def test_completion_colon_powershell_partial():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_powershell",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine",
            "_TYPER_COMPLETE_WORD_TO_COMPLETE": "alpine",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_powershell_single():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_powershell",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine:hell",
            "_TYPER_COMPLETE_WORD_TO_COMPLETE": "alpine:hell",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" not in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_pwsh_all():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_pwsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name",
        },
    )

    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" in result.stdout


def test_completion_colon_pwsh_partial():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_pwsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine",
            "_TYPER_COMPLETE_WORD_TO_COMPLETE": "alpine",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


def test_completion_colon_pwsh_single():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_COLON_EXAMPLE.PY_COMPLETE": "complete_pwsh",
            "_TYPER_COMPLETE_ARGS": "colon_example.py --name alpine:hell",
            "_TYPER_COMPLETE_WORD_TO_COMPLETE": "alpine:hell",
        },
    )
    assert "alpine:hello" in result.stdout
    assert "alpine:latest" not in result.stdout
    assert "nvidia/cuda:10.0-devel-ubuntu18.04" not in result.stdout


# TODO: tests for complete_fish



import typer
from typing_extensions import Annotated


def main(user_name: Annotated[str, typer.Option("--name", "-n")]):
    print(f"Hello {user_name}")


if __name__ == "__main__":
    typer.run(main)



from typing import List

import typer

app = typer.Typer()


@app.command()
def main(name: List[str] = typer.Option(["World"], help="The name to say hi to.")):
    for each_name in name:
        print(f"Hello {each_name}")


if __name__ == "__main__":
    app()



import typer

app = typer.Typer()


@app.command()
def create(username: str):
    print(f"Creating user: {username}")


@app.command()
def delete(username: str):
    print(f"Deleting user: {username}")


@app.callback()
def main(ctx: typer.Context):
    """
    Manage users in the awesome CLI app.
    """
    print(f"About to execute command: {ctx.invoked_subcommand}")


if __name__ == "__main__":
    app()



from enum import Enum
from typing import List

import typer


class Food(str, Enum):
    food_1 = "Eggs"
    food_2 = "Bacon"
    food_3 = "Cheese"


def main(groceries: List[Food] = typer.Option([Food.food_1, Food.food_3])):
    print(f"Buying groceries: {', '.join([f.value for f in groceries])}")


if __name__ == "__main__":
    typer.run(main)



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.arguments.help import tutorial003_an as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "[OPTIONS] [NAME]" in result.output
    assert "Say hi to NAME very gently, like Dirk." in result.output
    assert "Arguments" in result.output
    assert "NAME" in result.output
    assert "Who to greet" in result.output
    assert "[default: World]" in result.output


def test_call_arg():
    result = runner.invoke(app, ["Camila"])
    assert result.exit_code == 0
    assert "Hello Camila" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.multiple_values.multiple_options import tutorial001 as mod

runner = CliRunner()
app = typer.Typer()
app.command()(mod.main)


def test_main():
    result = runner.invoke(app)
    assert result.exit_code != 0
    assert "No provided users" in result.output
    assert "raw input = None" in result.output
    assert "Aborted" in result.output


def test_1_user():
    result = runner.invoke(app, ["--user", "Camila"])
    assert result.exit_code == 0
    assert "Processing user: Camila" in result.output


def test_3_user():
    result = runner.invoke(
        app, ["--user", "Camila", "--user", "Rick", "--user", "Morty"]
    )
    assert result.exit_code == 0
    assert "Processing user: Camila" in result.output
    assert "Processing user: Rick" in result.output
    assert "Processing user: Morty" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer
from typing_extensions import Annotated


def main(name: Annotated[str, typer.Argument(envvar="AWESOME_NAME")] = "World"):
    print(f"Hello Mr. {name}")


if __name__ == "__main__":
    typer.run(main)



import typer


def main(
    name: str,
    lastname: str = typer.Option("", help="Last name of person to greet."),
    formal: bool = typer.Option(
        False, help="Say hi formally.", rich_help_panel="Customization and Utils"
    ),
    debug: bool = typer.Option(
        False, help="Enable debugging.", rich_help_panel="Customization and Utils"
    ),
):
    """
    Say hi to NAME, optionally with a --lastname.

    If --formal is used, say hi very formally.
    """
    if formal:
        print(f"Good day Ms. {name} {lastname}.")
    else:
        print(f"Hello {name} {lastname}")


if __name__ == "__main__":
    typer.run(main)



import logging
import re

from github import Github
from pydantic import BaseModel, SecretStr
from pydantic_settings import BaseSettings

site_domain = "typer.tiangolo.com"


class Settings(BaseSettings):
    github_repository: str
    github_token: SecretStr
    deploy_url: str | None = None
    commit_sha: str
    run_id: int
    is_done: bool = False


class LinkData(BaseModel):
    previous_link: str
    preview_link: str


def main() -> None:
    logging.basicConfig(level=logging.INFO)
    settings = Settings()

    logging.info(f"Using config: {settings.model_dump_json()}")
    g = Github(settings.github_token.get_secret_value())
    repo = g.get_repo(settings.github_repository)
    use_pr = next(
        (pr for pr in repo.get_pulls() if pr.head.sha == settings.commit_sha), None
    )
    if not use_pr:
        logging.error(f"No PR found for hash: {settings.commit_sha}")
        return
    commits = list(use_pr.get_commits())
    current_commit = [c for c in commits if c.sha == settings.commit_sha][0]
    run_url = f"https://github.com/{settings.github_repository}/actions/runs/{settings.run_id}"
    if settings.is_done and not settings.deploy_url:
        current_commit.create_status(
            state="success",
            description="No Docs Changes",
            context="deploy-docs",
            target_url=run_url,
        )
        logging.info("No docs changes found")
        return
    if not settings.deploy_url:
        current_commit.create_status(
            state="pending",
            description="Deploying Docs",
            context="deploy-docs",
            target_url=run_url,
        )
        logging.info("No deploy URL available yet")
        return
    current_commit.create_status(
        state="success",
        description="Docs Deployed",
        context="deploy-docs",
        target_url=run_url,
    )

    files = list(use_pr.get_files())
    docs_files = [f for f in files if f.filename.startswith("docs/")]

    deploy_url = settings.deploy_url.rstrip("/")
    links: list[LinkData] = []
    for f in docs_files:
        match = re.match(r"docs/(.*)", f.filename)
        if not match:
            continue
        path = match.group(1)
        if path.endswith("index.md"):
            use_path = path.replace("index.md", "")
        else:
            use_path = path.replace(".md", "/")
        link = LinkData(
            previous_link=f"https://{site_domain}/{use_path}",
            preview_link=f"{deploy_url}/{use_path}",
        )
        links.append(link)
        links.sort(key=lambda x: x.preview_link)

    message = f"📝 Docs preview for commit {settings.commit_sha} at: {deploy_url}"

    if links:
        message += "\n\n### Modified Pages\n\n"
        for link in links:
            message += f"* {link.preview_link}"
            message += f" - ([before]({link.previous_link}))"
            message += "\n"

    print(message)
    use_pr.as_issue().create_comment(message)

    logging.info("Finished")


if __name__ == "__main__":
    main()



import os
import subprocess
import sys
from pathlib import Path
from unittest import mock

import shellingham
import typer
from typer.testing import CliRunner

from docs_src.commands.index import tutorial001 as mod

from ..utils import requires_completion_permission

runner = CliRunner()
app = typer.Typer()
app.command()(mod.main)


@requires_completion_permission
def test_completion_install_no_shell():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--install-completion"],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TYPER_COMPLETE_TEST_DISABLE_SHELL_DETECTION": "True",
        },
    )
    assert "Option '--install-completion' requires an argument" in result.stderr


@requires_completion_permission
def test_completion_install_bash():
    bash_completion_path: Path = Path.home() / ".bashrc"
    text = ""
    if bash_completion_path.is_file():
        text = bash_completion_path.read_text()
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            mod.__file__,
            "--install-completion",
            "bash",
        ],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TYPER_COMPLETE_TEST_DISABLE_SHELL_DETECTION": "True",
        },
    )
    new_text = bash_completion_path.read_text()
    bash_completion_path.write_text(text)
    install_source = Path(".bash_completions/tutorial001.py.sh")
    assert str(install_source) not in text
    assert str(install_source) in new_text
    assert "completion installed in" in result.stdout
    assert "Completion will take effect once you restart the terminal" in result.stdout
    install_source_path = Path.home() / install_source
    assert install_source_path.is_file()
    install_content = install_source_path.read_text()
    install_source_path.unlink()
    assert (
        "complete -o default -F _tutorial001py_completion tutorial001.py"
        in install_content
    )


@requires_completion_permission
def test_completion_install_zsh():
    completion_path: Path = Path.home() / ".zshrc"
    text = ""
    if not completion_path.is_file():  # pragma: no cover
        completion_path.write_text('echo "custom .zshrc"')
    if completion_path.is_file():
        text = completion_path.read_text()
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            mod.__file__,
            "--install-completion",
            "zsh",
        ],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TYPER_COMPLETE_TEST_DISABLE_SHELL_DETECTION": "True",
        },
    )
    new_text = completion_path.read_text()
    completion_path.write_text(text)
    zfunc_fragment = "fpath+=~/.zfunc"
    assert zfunc_fragment in new_text
    assert "completion installed in" in result.stdout
    assert "Completion will take effect once you restart the terminal" in result.stdout
    install_source_path = Path.home() / ".zfunc/_tutorial001.py"
    assert install_source_path.is_file()
    install_content = install_source_path.read_text()
    install_source_path.unlink()
    assert "compdef _tutorial001py_completion tutorial001.py" in install_content


@requires_completion_permission
def test_completion_install_fish():
    script_path = Path(mod.__file__)
    completion_path: Path = (
        Path.home() / f".config/fish/completions/{script_path.name}.fish"
    )
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            mod.__file__,
            "--install-completion",
            "fish",
        ],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_TYPER_COMPLETE_TEST_DISABLE_SHELL_DETECTION": "True",
        },
    )
    new_text = completion_path.read_text()
    completion_path.unlink()
    assert "complete --command tutorial001.py" in new_text
    assert "completion installed in" in result.stdout
    assert "Completion will take effect once you restart the terminal" in result.stdout


@requires_completion_permission
def test_completion_install_powershell():
    completion_path: Path = (
        Path.home() / ".config/powershell/Microsoft.PowerShell_profile.ps1"
    )
    completion_path_bytes = f"{completion_path}\n".encode("windows-1252")
    text = ""
    if completion_path.is_file():  # pragma: no cover
        text = completion_path.read_text()

    with mock.patch.object(
        shellingham, "detect_shell", return_value=("pwsh", "/usr/bin/pwsh")
    ):
        with mock.patch.object(
            subprocess,
            "run",
            return_value=subprocess.CompletedProcess(
                ["pwsh"], returncode=0, stdout=completion_path_bytes
            ),
        ):
            result = runner.invoke(app, ["--install-completion"])
    install_script = "Register-ArgumentCompleter -Native -CommandName mocked-typer-testing-app -ScriptBlock $scriptblock"
    parent: Path = completion_path.parent
    parent.mkdir(parents=True, exist_ok=True)
    completion_path.write_text(install_script)
    new_text = completion_path.read_text()
    completion_path.write_text(text)
    assert install_script not in text
    assert install_script in new_text
    assert "completion installed in" in result.stdout
    assert "Completion will take effect once you restart the terminal" in result.stdout



from pathlib import Path
from typing import List

import typer


def main(files: List[Path], celebration: str):
    for path in files:
        if path.is_file():
            print(f"This file exists: {path.name}")
            print(celebration)


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys
from pathlib import Path


def test_traceback_no_rich():
    file_path = Path(__file__).parent / "assets/type_error_no_rich.py"
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", str(file_path)],
        capture_output=True,
        encoding="utf-8",
        env={**os.environ, "_TYPER_STANDARD_TRACEBACK": ""},
    )
    assert "return get_command(self)(*args, **kwargs)" not in result.stderr

    assert "typer.run(main)" in result.stderr
    assert "print(name + 3)" in result.stderr
    assert 'TypeError: can only concatenate str (not "int") to str' in result.stderr


def test_traceback_no_rich_short_disable():
    file_path = Path(__file__).parent / "assets/type_error_no_rich_short_disable.py"
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", str(file_path)],
        capture_output=True,
        encoding="utf-8",
        env={**os.environ, "_TYPER_STANDARD_TRACEBACK": ""},
    )
    assert "return get_command(self)(*args, **kwargs)" not in result.stderr

    assert "app()" in result.stderr
    assert "print(name + 3)" in result.stderr
    assert 'TypeError: can only concatenate str (not "int") to str' in result.stderr


def test_unmodified_traceback():
    file_path = Path(__file__).parent / "assets/type_error_normal_traceback.py"
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", str(file_path)],
        capture_output=True,
        encoding="utf-8",
        env={**os.environ, "_TYPER_STANDARD_TRACEBACK": ""},
    )
    assert "morty" in result.stdout, "the call to the first app should work normally"
    assert "return callback(**use_params)" in result.stderr, (
        "calling outside of Typer should show the normal traceback, "
        "even after the hook is installed"
    )
    assert "typer.main.get_command(broken_app)()" in result.stderr
    assert "print(name + 3)" in result.stderr
    assert 'TypeError: can only concatenate str (not "int") to str' in result.stderr



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.options.prompt import tutorial003_an as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_prompt():
    result = runner.invoke(app, input="Old Project\nOld Project\n")
    assert result.exit_code == 0
    assert "Deleting project Old Project" in result.output


def test_prompt_not_equal():
    result = runner.invoke(
        app, input="Old Project\nNew Spice\nOld Project\nOld Project\n"
    )
    assert result.exit_code == 0
    assert "Error: The two entered values do not match" in result.output
    assert "Deleting project Old Project" in result.output


def test_option():
    result = runner.invoke(app, ["--project-name", "Old Project"])
    assert result.exit_code == 0
    assert "Deleting project Old Project" in result.output
    assert "Project name: " not in result.output


def test_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "--project-name" in result.output
    assert "TEXT" in result.output
    assert "[required]" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import subprocess
import sys

import pytest
import typer
import typer.core
from typer.testing import CliRunner

from docs_src.options.required import tutorial001 as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_1():
    result = runner.invoke(app, ["Camila"])
    assert result.exit_code != 0
    assert "Missing option '--lastname'" in result.output


def test_option_lastname():
    result = runner.invoke(app, ["Camila", "--lastname", "Gutiérrez"])
    assert result.exit_code == 0
    assert "Hello Camila Gutiérrez" in result.output


def test_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "--lastname" in result.output
    assert "TEXT" in result.output
    assert "[required]" in result.output


def test_help_no_rich(monkeypatch: pytest.MonkeyPatch):
    monkeypatch.setattr(typer.core, "HAS_RICH", False)
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "--lastname" in result.output
    assert "TEXT" in result.output
    assert "[required]" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer


def main():
    delete = typer.confirm("Are you sure you want to delete it?")
    if not delete:
        print("Not deleting")
        raise typer.Abort()
    print("Deleting it!")


if __name__ == "__main__":
    typer.run(main)



import click
import typer

app = typer.Typer()


def shell_complete(ctx: click.Context, param: click.Parameter, incomplete: str):
    typer.echo(f"ctx: {ctx.info_name}", err=True)
    typer.echo(f"arg is: {param.name}", err=True)
    typer.echo(f"incomplete is: {incomplete}", err=True)
    return ["Emma"]


@app.command(context_settings={"auto_envvar_prefix": "TEST"})
def main(name: str = typer.Argument(shell_complete=shell_complete)):
    """
    Say hello.
    """


if __name__ == "__main__":
    app()



import typer
from typing_extensions import Annotated


def main(
    name: Annotated[str, typer.Argument(help="Who to greet")],
    lastname: Annotated[
        str, typer.Argument(help="The last name", rich_help_panel="Secondary Arguments")
    ] = "",
    age: Annotated[
        str,
        typer.Argument(help="The user's age", rich_help_panel="Secondary Arguments"),
    ] = "",
):
    """
    Say hi to NAME very gently, like Dirk.
    """
    print(f"Hello {name}")


if __name__ == "__main__":
    typer.run(main)



import subprocess
import sys

import typer
from typer.testing import CliRunner

from docs_src.arguments.help import tutorial002 as mod

runner = CliRunner()

app = typer.Typer()
app.command()(mod.main)


def test_help():
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "[OPTIONS] NAME" in result.output
    assert "Say hi to NAME very gently, like Dirk." in result.output
    assert "Arguments" in result.output
    assert "NAME" in result.output
    assert "The name of the user to greet" in result.output
    assert "[required]" in result.output


def test_call_arg():
    result = runner.invoke(app, ["Camila"])
    assert result.exit_code == 0
    assert "Hello Camila" in result.output


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer

app = typer.Typer(rich_markup_mode="rich")


@app.command()
def create(username: str):
    """
    [green]Create[/green] a new user. :sparkles:
    """
    print(f"Creating user: {username}")


@app.command()
def delete(username: str):
    """
    [red]Delete[/red] a user. :fire:
    """
    print(f"Deleting user: {username}")


@app.command(rich_help_panel="Utils and Configs")
def config(configuration: str):
    """
    [blue]Configure[/blue] the system. :wrench:
    """
    print(f"Configuring the system with: {configuration}")


@app.command(rich_help_panel="Utils and Configs")
def sync():
    """
    [blue]Synchronize[/blue] the system or something fancy like that. :recycle:
    """
    print("Syncing the system")


@app.command(rich_help_panel="Help and Others")
def help():
    """
    Get [yellow]help[/yellow] with the system. :question:
    """
    print("Opening help portal...")


@app.command(rich_help_panel="Help and Others")
def report():
    """
    [yellow]Report[/yellow] an issue. :bug:
    """
    print("Please open a new issue online, not a direct message")


if __name__ == "__main__":
    app()



import typer
from typing_extensions import Annotated


def main(
    project_name: Annotated[str, typer.Option(prompt=True, confirmation_prompt=True)],
):
    print(f"Deleting project {project_name}")


if __name__ == "__main__":
    typer.run(main)



import errno

import typer
import typer.completion
from typer.testing import CliRunner

runner = CliRunner()


def test_eoferror():
    # Mainly for coverage/completeness
    app = typer.Typer()

    @app.command()
    def main():
        raise EOFError()

    result = runner.invoke(app)
    assert result.exit_code == 1


def test_keyboardinterrupt():
    # Mainly for coverage/completeness
    app = typer.Typer()

    @app.command()
    def main():
        raise KeyboardInterrupt()

    result = runner.invoke(app)
    assert result.exit_code == 130
    assert result.stdout == ""


def test_oserror():
    # Mainly for coverage/completeness
    app = typer.Typer()

    @app.command()
    def main():
        e = OSError()
        e.errno = errno.EPIPE
        raise e

    result = runner.invoke(app)
    assert result.exit_code == 1


def test_oserror_no_epipe():
    # Mainly for coverage/completeness
    app = typer.Typer()

    @app.command()
    def main():
        raise OSError()

    result = runner.invoke(app)
    assert result.exit_code == 1



import typer
from typing_extensions import Annotated


def main(
    id: Annotated[int, typer.Argument(min=0, max=1000)],
    rank: Annotated[int, typer.Option(max=10, clamp=True)] = 0,
    score: Annotated[float, typer.Option(min=0, max=100, clamp=True)] = 0,
):
    print(f"ID is {id}")
    print(f"--rank is {rank}")
    print(f"--score is {score}")


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys

from . import example_rich_tags as mod


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "create", "DeadPool"],
        capture_output=True,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "Creating user: DeadPool" in result.stdout

    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "delete", "DeadPool"],
        capture_output=True,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "Deleting user: DeadPool" in result.stdout

    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "delete-all"],
        capture_output=True,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "Deleting all users" in result.stdout


def test_completion_complete_subcommand_bash():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_EXAMPLE_RICH_TAGS.PY_COMPLETE": "complete_bash",
            "COMP_WORDS": "example_rich_tags.py del",
            "COMP_CWORD": "1",
        },
    )
    assert "delete\ndelete-all" in result.stdout


def test_completion_complete_subcommand_zsh():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_EXAMPLE_RICH_TAGS.PY_COMPLETE": "complete_zsh",
            "_TYPER_COMPLETE_ARGS": "example_rich_tags.py del",
        },
    )
    assert (
        """_arguments '*: :(("delete":"Delete a user with USERNAME."\n"""
        """\"delete-all":"Delete ALL users in the database."))'"""
    ) in result.stdout


def test_completion_complete_subcommand_fish():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_EXAMPLE_RICH_TAGS.PY_COMPLETE": "complete_fish",
            "_TYPER_COMPLETE_ARGS": "example_rich_tags.py del",
            "_TYPER_COMPLETE_FISH_ACTION": "get-args",
        },
    )
    assert (
        "delete\tDelete a user with USERNAME.\ndelete-all\tDelete ALL users in the database."
        in result.stdout
    )


def test_completion_complete_subcommand_powershell():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_EXAMPLE_RICH_TAGS.PY_COMPLETE": "complete_powershell",
            "_TYPER_COMPLETE_ARGS": "example_rich_tags.py del",
        },
    )
    assert (
        "delete:::Delete a user with USERNAME.\ndelete-all:::Delete ALL users in the database."
    ) in result.stdout


def test_completion_complete_subcommand_pwsh():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, " "],
        capture_output=True,
        encoding="utf-8",
        env={
            **os.environ,
            "_EXAMPLE_RICH_TAGS.PY_COMPLETE": "complete_pwsh",
            "_TYPER_COMPLETE_ARGS": "example_rich_tags.py del",
        },
    )
    assert (
        "delete:::Delete a user with USERNAME.\ndelete-all:::Delete ALL users in the database."
    ) in result.stdout



import subprocess
import sys

from docs_src.testing.app02 import main as mod
from docs_src.testing.app02.test_main import test_app


def test_app02():
    test_app()


def test_script():
    result = subprocess.run(
        [sys.executable, "-m", "coverage", "run", mod.__file__, "--help"],
        capture_output=True,
        encoding="utf-8",
    )
    assert "Usage" in result.stdout



import typer
from typing_extensions import Annotated


def main(config: Annotated[typer.FileText, typer.Option()]):
    for line in config:
        print(f"Config line: {line}")


if __name__ == "__main__":
    typer.run(main)



import pytest
import typer
from typer.testing import CliRunner
from typer.utils import (
    AnnotatedParamWithDefaultValueError,
    DefaultFactoryAndDefaultValueError,
    MixedAnnotatedAndDefaultStyleError,
    MultipleTyperAnnotationsError,
    _split_annotation_from_typer_annotations,
)
from typing_extensions import Annotated

runner = CliRunner()


def test_split_annotations_from_typer_annotations_simple():
    # Simple sanity check that this utility works. If this isn't working on a given
    # python version, then no other tests for Annotated will work.
    given = Annotated[str, typer.Argument()]
    base, typer_annotations = _split_annotation_from_typer_annotations(given)
    assert base is str
    # No equality check on the param types. Checking the length is sufficient.
    assert len(typer_annotations) == 1


def test_forbid_default_value_in_annotated_argument():
    app = typer.Typer()

    # This test case only works with `typer.Argument`. `typer.Option` uses positionals
    # for param_decls too.
    @app.command()
    def cmd(my_param: Annotated[str, typer.Argument("foo")]): ...  # pragma: no cover

    with pytest.raises(AnnotatedParamWithDefaultValueError) as excinfo:
        runner.invoke(app)

    assert vars(excinfo.value) == {
        "param_type": typer.models.ArgumentInfo,
        "argument_name": "my_param",
    }


def test_allow_options_to_have_names():
    app = typer.Typer()

    @app.command()
    def cmd(my_param: Annotated[str, typer.Option("--some-opt")]):
        print(my_param)

    result = runner.invoke(app, ["--some-opt", "hello"])
    assert result.exit_code == 0, result.output
    assert "hello" in result.output


@pytest.mark.parametrize(
    ["param", "param_info_type"],
    [
        (typer.Argument, typer.models.ArgumentInfo),
        (typer.Option, typer.models.OptionInfo),
    ],
)
def test_forbid_annotated_param_and_default_param(param, param_info_type):
    app = typer.Typer()

    @app.command()
    def cmd(my_param: Annotated[str, param()] = param("foo")): ...  # pragma: no cover

    with pytest.raises(MixedAnnotatedAndDefaultStyleError) as excinfo:
        runner.invoke(app)

    assert vars(excinfo.value) == {
        "argument_name": "my_param",
        "annotated_param_type": param_info_type,
        "default_param_type": param_info_type,
    }


def test_forbid_multiple_typer_params_in_annotated():
    app = typer.Typer()

    @app.command()
    def cmd(
        my_param: Annotated[str, typer.Argument(), typer.Argument()],
    ): ...  # pragma: no cover

    with pytest.raises(MultipleTyperAnnotationsError) as excinfo:
        runner.invoke(app)

    assert vars(excinfo.value) == {"argument_name": "my_param"}


def test_allow_multiple_non_typer_params_in_annotated():
    app = typer.Typer()

    @app.command()
    def cmd(my_param: Annotated[str, "someval", typer.Argument(), 4] = "hello"):
        print(my_param)

    result = runner.invoke(app)
    # Should behave like normal
    assert result.exit_code == 0, result.output
    assert "hello" in result.output


@pytest.mark.parametrize(
    ["param", "param_info_type"],
    [
        (typer.Argument, typer.models.ArgumentInfo),
        (typer.Option, typer.models.OptionInfo),
    ],
)
def test_forbid_default_factory_and_default_value_in_annotated(param, param_info_type):
    def make_string():
        return "foo"  # pragma: no cover

    app = typer.Typer()

    @app.command()
    def cmd(
        my_param: Annotated[str, param(default_factory=make_string)] = "hello",
    ): ...  # pragma: no cover

    with pytest.raises(DefaultFactoryAndDefaultValueError) as excinfo:
        runner.invoke(app)

    assert vars(excinfo.value) == {
        "argument_name": "my_param",
        "param_type": param_info_type,
    }


@pytest.mark.parametrize(
    "param",
    [
        typer.Argument,
        typer.Option,
    ],
)
def test_allow_default_factory_with_default_param(param):
    def make_string():
        return "foo"

    app = typer.Typer()

    @app.command()
    def cmd(my_param: str = param(default_factory=make_string)):
        print(my_param)

    result = runner.invoke(app)
    assert result.exit_code == 0, result.output
    assert "foo" in result.output


@pytest.mark.parametrize(
    ["param", "param_info_type"],
    [
        (typer.Argument, typer.models.ArgumentInfo),
        (typer.Option, typer.models.OptionInfo),
    ],
)
def test_forbid_default_and_default_factory_with_default_param(param, param_info_type):
    def make_string():
        return "foo"  # pragma: no cover

    app = typer.Typer()

    @app.command()
    def cmd(
        my_param: str = param("hi", default_factory=make_string),
    ): ...  # pragma: no cover

    with pytest.raises(DefaultFactoryAndDefaultValueError) as excinfo:
        runner.invoke(app)

    assert vars(excinfo.value) == {
        "argument_name": "my_param",
        "param_type": param_info_type,
    }


@pytest.mark.parametrize(
    ["error", "message"],
    [
        (
            AnnotatedParamWithDefaultValueError(
                argument_name="my_argument",
                param_type=typer.models.ArgumentInfo,
            ),
            "`Argument` default value cannot be set in `Annotated` for 'my_argument'. Set the default value with `=` instead.",
        ),
        (
            MixedAnnotatedAndDefaultStyleError(
                argument_name="my_argument",
                annotated_param_type=typer.models.OptionInfo,
                default_param_type=typer.models.ArgumentInfo,
            ),
            "Cannot specify `Option` in `Annotated` and `Argument` as a default value together for 'my_argument'",
        ),
        (
            MixedAnnotatedAndDefaultStyleError(
                argument_name="my_argument",
                annotated_param_type=typer.models.OptionInfo,
                default_param_type=typer.models.OptionInfo,
            ),
            "Cannot specify `Option` in `Annotated` and default value together for 'my_argument'",
        ),
        (
            MixedAnnotatedAndDefaultStyleError(
                argument_name="my_argument",
                annotated_param_type=typer.models.ArgumentInfo,
                default_param_type=typer.models.ArgumentInfo,
            ),
            "Cannot specify `Argument` in `Annotated` and default value together for 'my_argument'",
        ),
        (
            MultipleTyperAnnotationsError(
                argument_name="my_argument",
            ),
            "Cannot specify multiple `Annotated` Typer arguments for 'my_argument'",
        ),
        (
            DefaultFactoryAndDefaultValueError(
                argument_name="my_argument",
                param_type=typer.models.OptionInfo,
            ),
            "Cannot specify `default_factory` and a default value together for `Option`",
        ),
    ],
)
def test_error_rendering(error, message):
    assert str(error) == message



import click
import typer

app = typer.Typer()


@app.command()
def top():
    """
    Top level command, form Typer
    """
    print("The Typer app is at the top level")


@app.callback()
def callback():
    """
    Typer app, including Click subapp
    """


@click.command()
@click.option("--name", prompt="Your name", help="The person to greet.")
def hello(name):
    """Simple program that greets NAME for a total of COUNT times."""
    click.echo(f"Hello {name}!")


typer_click_object = typer.main.get_command(app)

typer_click_object.add_command(hello, "hello")

if __name__ == "__main__":
    typer_click_object()



import typer
from rich import print

data = {
    "name": "Rick",
    "age": 42,
    "items": [{"name": "Portal Gun"}, {"name": "Plumbus"}],
    "active": True,
    "affiliation": None,
}


def main():
    print("Here's the data")
    print(data)


if __name__ == "__main__":
    typer.run(main)



import os
import subprocess
import sys

import pytest
from typer.testing import CliRunner

from docs_src.subcommands import tutorial001

runner = CliRunner()


@pytest.fixture()
def mod(monkeypatch):
    with monkeypatch.context():
        monkeypatch.syspath_prepend(list(tutorial001.__path__)[0])
        from docs_src.subcommands.tutorial001 import main

        return main


@pytest.fixture()
def app(mod):
    return mod.app


def test_help(app):
    result = runner.invoke(app, ["--help"])
    assert result.exit_code == 0
    assert "[OPTIONS] COMMAND [ARGS]..." in result.output
    assert "Commands" in result.output
    assert "items" in result.output
    assert "users" in result.output


def test_help_items(app):
    result = runner.invoke(app, ["items", "--help"])
    assert result.exit_code == 0
    assert "[OPTIONS] COMMAND [ARGS]..." in result.output
    assert "Commands" in result.output
    assert "create" in result.output
    assert "delete" in result.output
    assert "sell" in result.output


def test_items_create(app):
    result = runner.invoke(app, ["items", "create", "Wand"])
    assert result.exit_code == 0
    assert "Creating item: Wand" in result.output


def test_items_sell(app):
    result = runner.invoke(app, ["items", "sell", "Vase"])
    assert result.exit_code == 0
    assert "Selling item: Vase" in result.output


def test_items_delete(app):
    result = runner.invoke(app, ["items", "delete", "Vase"])
    assert result.exit_code == 0
    assert "Deleting item: Vase" in result.output


def test_help_users(app):
    result = runner.invoke(app, ["users", "--help"])
    assert result.exit_code == 0
    assert "[OPTIONS] COMMAND [ARGS]..." in result.output
    assert "Commands" in result.output
    assert "create" in result.output
    assert "delete" in result.output
    assert "sell" not in result.output


def test_users_create(app):
    result = runner.invoke(app, ["users", "create", "Camila"])
    assert result.exit_code == 0
    assert "Creating user: Camila" in result.output


def test_users_delete(app):
    result = runner.invoke(app, ["users", "delete", "Camila"])
    assert result.exit_code == 0
    assert "Deleting user: Camila" in result.output


def test_scripts(mod):
    from docs_src.subcommands.tutorial001 import items, users

    env = os.environ.copy()
    env["PYTHONPATH"] = ":".join(list(tutorial001.__path__))

    for module in [mod, items, users]:
        result = subprocess.run(
            [sys.executable, "-m", "coverage", "run", module.__file__, "--help"],
            capture_output=True,
            encoding="utf-8",
            env=env,
        )
        assert "Usage" in result.stdout



import typer

app = typer.Typer()


@app.callback()
def callback():
    """
    Manage users CLI app.

    Use it with the create command.

    A new user with the given NAME will be created.
    """


@app.command()
def create(name: str):
    print(f"Creating user: {name}")


if __name__ == "__main__":
    app()



import typer


def main(
    name: str,
    lastname: str = typer.Option("", help="Last name of person to greet."),
    formal: bool = typer.Option(False, help="Say hi formally."),
):
    """
    Say hi to NAME, optionally with a --lastname.

    If --formal is used, say hi very formally.
    """
    if formal:
        print(f"Good day Ms. {name} {lastname}.")
    else:
        print(f"Hello {name} {lastname}")


if __name__ == "__main__":
    typer.run(main)



import subprocess
import sys


def test_script_help():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "--help",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "top" in result.stdout
    assert "hello" in result.stdout
    assert "sub" in result.stdout


def test_script_top():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "top",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "top" in result.stdout


def test_script_hello():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "hello",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "hello there" in result.stdout


def test_script_bye():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "bye",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "bye" in result.stdout


def test_script_sub_command_help():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "sub",
            "--help",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "sub-sub-command" in result.stdout


def test_script_sub_sub_command():
    result = subprocess.run(
        [
            sys.executable,
            "-m",
            "coverage",
            "run",
            "-m",
            "typer",
            "tests/assets/cli/extended_app_cli.py",
            "run",
            "sub",
            "sub-sub-command",
        ],
        capture_output=True,
        encoding="utf-8",
    )
    assert "sub_sub_command" in result.stdout



import typer
from typing_extensions import Annotated


def main(
    name: Annotated[
        str, typer.Argument(envvar="AWESOME_NAME", show_envvar=False)
    ] = "World",
):
    print(f"Hello Mr. {name}")


if __name__ == "__main__":
    typer.run(main)
